﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Devices;
using Produmex.Sbo.Logex.Data.Generators.BatchNumberGenerators;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;

namespace Produmex.Sbo.Logex.WorkflowScripts
{
	public class WorkflowScript_AfterItemPickedHookScript : WorkflowInstanceScriptBase
	{
		private static class Definitions
		{
			public static Collection<string> SHIPPING_TYPE = new Collection<string>(new string[] { "PostNL", "SIS" });
			public const string INITIATOR_CODE = "120005";

			public const string REPORT_PATH_RETOUR_LABEL = @"MoviantoRetourLabel.rpt";
			public const string REPORT_PATH_NAME_LABEL = @"MoviantoNameLabel.rpt";
			public const int NUMBER_OF_LABELS = 1;

		}

		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		// Input parameters * do not change *
		public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
		public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

		public ReadOnlyBinder<Collection<PickingItemInfo>> PickedItemInfos;
		public ReadOnlyBinder<int?> LUID;
		public ReadOnlyBinder<Collection<string>> ListOfSerialNumbers;
		public ReadOnlyBinder<int> DocEntry;
		public ReadOnlyBinder<string> ObjType;

		// Sub-flows
		// <none>


		public WorkflowScript_AfterItemPickedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
			: base(parent, factory)
		{
		}

		#region WorkflowInstanceScriptBase Members

		protected override void Execute()
		{
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
			Message msg = null;


			//First check if we need to perform actions

			if (ListOfSerialNumbers == null ||
				ListOfSerialNumbers.Get() == null ||
				ListOfSerialNumbers.Get().Count == 0)
			{
				//No serial numbers, so do nothing
				return;
			}

			if (ObjType == null ||
			   ObjType.Get() == null ||
			   ObjType.Get() != Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxWave)
			{
				//Not a wave, so do nothing
				return;
			}

			if (PickedItemInfos == null ||
			   PickedItemInfos.Get() == null ||
			   PickedItemInfos.Get().Count == 0)
			{
				//No pick list object
				return;
			}

			int waveKey = DocEntry.Get();

			int? pickListDocEntry = PickedItemInfos.Get()[0].PickListDocEntry;
			int? pickListLineNum = PickedItemInfos.Get()[0].PickListLineID;

        
            string query = BuildQuery.GetShippingInfoForPickList(waveKey, null, null, sboProviderService.GetDbTool());
			DataSet ds = sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(ds) == 0)
			{
				//Not shippingtype 'PostNL' or 'SIS' or not correct initiator
				return;
			}

		Step_PrintLabels:

			#region PRINT LABELS

			string trackingNumber = ds.Tables[0].Rows[0][Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TrackingNumberMovianto.NAME] as string;

			foreach (string serialNumber in ListOfSerialNumbers.Get())
			{
				//      string reportPathRetourLabel = Definitions.REPORT_PATH_RETOUR_LABEL;

				//    try
				//    {
				//print label
				//      sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintRetourLabel), trackingNumber, serialNumber, session.DeviceInstanceId, reportPathRetourLabel);

				//    Thread.Sleep(750);

				// }
				//catch (Exception ex)
				// {
				//     s_log.Error(ex.ToString());
				//    throw ex;
				// }

				if (ListOfSerialNumbers.Get().Count > 1 ||
					GetNumberOfRows(ds) > 1)
				{
					//Need to print additional report
					string reportPathNameLabel = Definitions.REPORT_PATH_NAME_LABEL;

					//Query the exact data
                    query = BuildQuery.GetShippingInfoForPickList(waveKey, pickListDocEntry, pickListLineNum, sboProviderService.GetDbTool());
					DataSet dsExact = sboProviderService.RunView(false, null, null, query);

					if (this.GetNumberOfRows(dsExact) > 0)
					{
						string contactPerson = dsExact.Tables[0].Rows[0][Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderLineTable.Columns.CommentsLine.NAME] as string;

						if (!contactPerson.Contains("**"))
						{
							contactPerson = "";
						}
						else
						{
							//Keep the text after '**'
							contactPerson = contactPerson.Split(new string[1] { "**" }, StringSplitOptions.None)[1];
						}

						try
						{
							//print label
							sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintNameLabel), contactPerson, serialNumber, session.DeviceInstanceId, reportPathNameLabel);

							Thread.Sleep(750);

						}
						catch (Exception ex)
						{
							s_log.Error(ex.ToString());
							throw ex;
						}
					}
					else
					{
						s_log.ErrorFormat("Name label: No data found for pick list {0} and line num {1}.", pickListDocEntry, pickListLineNum);
					}

				}

			}

			if (ListOfSerialNumbers.Get().Count > 1 ||
				GetNumberOfRows(ds) > 1)
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						  DefaultCultureInfo.Get(), BuildParamCollection(
							  "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
							  "TitleKey", "MSG_TITLE_REPRINT_LABEL"));

				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_PrintLabels;
				}
			}

			#endregion

		}

		#endregion

		#region HELPERS

		/// <summary>
		/// Executes the print label.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOsePrinter ExecutePrintRetourLabel(PmxDbConnection conn, object[] parameters)
		{
			PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

			Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
			reportParameters.Add(new ReportParameter("@trackingNumber", (string)parameters[0]));
			reportParameters.Add(new ReportParameter("@serialNumber", (string)parameters[1]));

			return prov.PrintReport(reportParameters, (string)parameters[3], Definitions.NUMBER_OF_LABELS, (string)parameters[2], null, false);
		}

		/// <summary>
		/// Executes the print label.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOsePrinter ExecutePrintNameLabel(PmxDbConnection conn, object[] parameters)
		{
			PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

			Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
			reportParameters.Add(new ReportParameter("@contactPerson", (string)parameters[0]));
			reportParameters.Add(new ReportParameter("@serialNumber", (string)parameters[1]));

			return prov.PrintReport(reportParameters, (string)parameters[3], Definitions.NUMBER_OF_LABELS, (string)parameters[2], null, false);
		}

		/// <summary>
		/// Gets the number of rows.
		/// </summary>
		/// <param name="ds">The ds.</param>
		/// <returns></returns>
		private int GetNumberOfRows(DataSet ds)
		{
			if (ds != null && ds.Tables.Count > 0)
			{
				return ds.Tables[0].Rows.Count;
			}
			return 0;
		}
		#endregion

		#region QUERY

		/// <summary>
		/// Static class grouping our native queries
		/// </summary>
		private class BuildQuery
		{


            public static string GetShippingInfoForPickList(int wavekey, int? pickListDocEntry, int? pickListLineNum, DbTool dbtool)
			{
                DbQueryBuilder strBuilder = dbtool.GetQueryBuilder();

                
                // StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);


				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TrackingNumberMovianto.NAME);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderLineTable.Columns.CommentsLine.NAME);


				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));


				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME);

				strBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseLine.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineNum.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(SboSalesOrderLineTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.DocEntry.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME);
				strBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.LineNum.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseLine.NAME);
				strBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.ObjectType.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseType.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(SboSalesOrderTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocEntry.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME);


				strBuilder.Append(" LEFT OUTER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxShippingTypeTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.TrnspCode.NAME);



				strBuilder.Append(" WHERE ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.WaveKey.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(wavekey, CultureInfo.InvariantCulture));

				if (Definitions.SHIPPING_TYPE != null &&
					Definitions.SHIPPING_TYPE.Count > 0)
				{
					strBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(strBuilder, PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.Name.NAME);
					strBuilder.Append(" IN ( ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(Definitions.SHIPPING_TYPE, CultureInfo.InvariantCulture));
					strBuilder.Append(" ) ");
				}

				if (!string.IsNullOrEmpty(Definitions.INITIATOR_CODE))
				{
					strBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.InitiatorCode.NAME);
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(Definitions.INITIATOR_CODE, CultureInfo.InvariantCulture));
				}

				if (pickListDocEntry.HasValue)
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.DocEntry.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry.Value, CultureInfo.InvariantCulture));
				}

				if (pickListLineNum.HasValue)
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListLineTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListLineTable.Columns.LineNum.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListLineNum.Value, CultureInfo.InvariantCulture));
				}

				return strBuilder.ToString();
			}


		}

		#endregion


	}


}
