﻿using System;
using System.Diagnostics;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using System.Xml;
using System.Web;
using System.Net;
using System.IO;
using System.Threading;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DataSetClient;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;      
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Sbo.Logex.MoviantoNL.Providers;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.MoviantoNL.ViewObjects.Definitions.Views;



// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
	/// <summary>
	/// Cycle counting
	/// </summary>
	public class WorkflowScript_AlternatePackagingMoviantoScript : WorkflowInstanceScriptBase
	{


		private static Collection<string> LIST_OF_OPDRACHTGEVERS = new Collection<string>();
		private static Collection<string> LIST_OF_NOT_PRINTED_SHIPPING_TYPES = new Collection<string>();


		private static void FillListOfInitiators()
		{
			LIST_OF_OPDRACHTGEVERS.Add("3230");
			LIST_OF_OPDRACHTGEVERS.Add("3230Test");
			LIST_OF_NOT_PRINTED_SHIPPING_TYPES.Add("GLS");
			LIST_OF_NOT_PRINTED_SHIPPING_TYPES.Add("POSTNL");
			LIST_OF_NOT_PRINTED_SHIPPING_TYPES.Add("SIS");
		}

		private static class Definitions
		{
			//report definitions

			public const string REPORT_PATH_LABEL = @"DirkRaesShippingLabelColliNL.rpt";
			public const string REPORT_PATH_SPECIFIC_LABEL = @"DirkRaesLabel GSK.rpt";
			public const int NUMBER_OF_LABELS = 1;
			public const string REPORT_PATH_LABEL_TRANS_O_FLEX = @"DirkRaesShippingLabelColliNL.rpt";
			public const string TRANS_O_FLEX_SHIPPING_TYPE_NAME = "TRANS-O-FLEX";
		}

		// Input parameters
		public ReadOnlyBinder<CultureInfo>   DefaultCultureInfo;
		public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		private ISboProviderService m_sboProviderService;


		//sales delivery created hook flow
		private WorkflowInstanceBase m_salesDeliveryCreatedHookFlow;
		private DocumentRef[] m_documentRefsSalesDeliveryCreatedHookFlow;


		#region Constructors

		public WorkflowScript_AlternatePackagingMoviantoScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory): base(parent, factory)
		{
			FillListOfInitiators();
		}

		#endregion

		#region WorkflowInstanceScriptBase Members

		/// <summary>
		/// Executes this instance.
		/// </summary>
		protected override void Execute()
		{
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
			ProdumexError pmxError = null;

			string initialErrorKey = null;
			string deviceID = session.DeviceInstanceId;
			SboUserInfo userInfo = m_sboProviderService.GetCurrentUser();
			string query = null;
			Message msg = null;




			m_salesDeliveryCreatedHookFlow = Create("SalesDeliveryCreatedHookScript");
			m_salesDeliveryCreatedHookFlow.BindParameters(
				new ReadOnlyBinder<CultureInfo>("DefaultCultureInfo", DefaultCultureInfo.Get()),
				new ReadOnlyBinder<PmxOseCompany>("PmxOseCompany", PmxOseCompany.Get()),
				new ReadOnlyBinder<DocumentRef[]>("DocumentRefs", this, "m_documentRefsSalesDeliveryCreatedHookFlow")
				);


			int pickListDocEntry = 0;

			double numberOfColliDouble = 0;
			int numberOfColli = 0;
			DataSet dsItems = null;
			string logisticCarrierCode = null;
			int luid = 0;
			DataSet dsLuid = null;
			DataSet dsOpdrachtgevers = null;
			DataSet dsCovidien = null;
			DataSet dsShippingType = null;
			Dictionary<string, object> deliveryData = null;

			DocumentRef[] salesDeliveries = null;

        Step_SelectPickList:

			#region SELECT PICK LIST

			pickListDocEntry   = 0;
            msg = null;

			query = BuildQuery.GetPickListForPacking(m_sboProviderService.GetDbTool());
            DataSet dsPickList = m_sboProviderService.RunView(false, null, null, query);

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Picking.ISelectPickListScreen),
					   DefaultCultureInfo.Get(), BuildParamCollection(
					   "TitleKey", "Select Picklist",
					"PickListDS", dsPickList,
					"InitialErrorKey", null));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".PickListSelected"))
            {
                pickListDocEntry = ExtractParameter<int>(msg.Parameters, "pickListDocEntry");
                
                DataRow[] rows = dsPickList.Tables[0].Select(string.Format("{0} = {1}", PickListDataSet.Columns.DOC_ENTRY.Name, pickListDocEntry));

                if (rows.Length > 0)
                {
					deliveryData = new Dictionary<string, object>();
                }
                else
                {
					goto Step_SelectPickList;
                }

            }
            else if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectPickList;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
				return;
			}

			#endregion

		Step_EnterNumberOfColli:

			#region ENTER NUMBER OF COLLI

			numberOfColli = 0;
			numberOfColliDouble = 0;



			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
			   "InitialErrorKey", initialErrorKey,
			   "TitleKey", "MSG_TITLE_ENTER_NUMBER_OF_COLLI",
			   "InitialQuantity", 1,
			   "NumberOfDigitsAfterDecimal", 0,
			   "MaximumQuantity", -1
			   ));
			msg = WaitForMessage();

			initialErrorKey = null;


			if (msg.Name.EndsWith(".QuantityEntered"))
			{
				numberOfColliDouble = ExtractParameter<double>(msg.Parameters, "quantity");
				goto Step_SelectLogisticCarrier;
			}

			//go back
			goto Step_SelectPickList;



			#endregion

		Step_SelectLogisticCarrier:

			#region SELECT LOGISTIC CARRIER

			logisticCarrierCode = null;

			//no items in list, so run the query again
			query = BuildQuery.GetProducts(null, null, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Sales, true, m_sboProviderService.GetDbTool());
			dsItems = m_sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsItems) > 0)
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"TitleKey", "MSG_TITLE_SELECT_LOGISTIC_CARRIER",
						"ProductDS", dsItems
						));
				msg = WaitForMessage();

				//clear data
				initialErrorKey = null;

				if (msg.Name.EndsWith(".ProductSelected"))
				{
					logisticCarrierCode = ExtractParameter<string>(msg.Parameters, "itemCode");

					//fill return value
					goto Step_FillReturnValue;
				}

				goto Step_EnterNumberOfColli;
			}
			else
			{
				//No logistic carriers in the system, so only fill in the number of colli
				goto Step_FillReturnValue;
			}

			#endregion

		Step_FillReturnValue:

			#region FILL RETURN VALUE

			numberOfColli = Convert.ToInt32(numberOfColliDouble);
			deliveryData.Add(PmxSalesDeliveryTable.Columns.Colli.NAME, numberOfColli);

			if (!string.IsNullOrEmpty(logisticCarrierCode))
			{
				deliveryData.Add(PmxSalesDeliveryTable.Columns.LogisticCarrierCode.NAME, logisticCarrierCode);
			}

			goto Step_PrintLabel;

			#endregion

		Step_PrintLabel:

			#region PRINT LABEL

			try
			{

				string reportPath = Definitions.REPORT_PATH_LABEL;

				//get the luid
				query = BuildQuery.GetLUID(pickListDocEntry);
				dsLuid = m_sboProviderService.RunView(false, null, null, query);

				if (this.GetNumberOfRows(dsLuid) > 0)
				{
					luid = Convert.ToInt32(dsLuid.Tables[0].Rows[0][PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME]);
				}


				//get the shipping type
				query = BuildQuery.GetShippingTypeForPickList(pickListDocEntry);
				dsShippingType = m_sboProviderService.RunView(false, null, null, query);

				if (this.GetNumberOfRows(dsShippingType) > 0)
				{
					if (Convert.ToString(dsShippingType.Tables[0].Rows[0][PmxShippingTypeTable.Columns.Name.NAME]).ToLower() == Definitions.TRANS_O_FLEX_SHIPPING_TYPE_NAME.ToLower())
					{
						reportPath = Definitions.REPORT_PATH_LABEL_TRANS_O_FLEX;
					}
				}





				//check if the specific label needs to be printed
				query = BuildQuery.GetOrderInfo(luid);
				dsOpdrachtgevers = m_sboProviderService.RunView(false, null, null, query);

				if (this.GetNumberOfRows(dsOpdrachtgevers) > 0)
				{
					//we found data, so now we check if the initiator is in the list
					if (LIST_OF_OPDRACHTGEVERS.Contains(Convert.ToString(dsOpdrachtgevers.Tables[0].Rows[0][PmxSalesDeliveryTable.Columns.InitiatorCode.NAME])))
					{
						reportPath = Definitions.REPORT_PATH_SPECIFIC_LABEL;
					}
				}

				// TBA Added

				query = BuildQuery.CheckCovidien(luid);
				dsCovidien = m_sboProviderService.RunView(false, null, null, query);
				if (this.GetNumberOfRows(dsCovidien) == 0)
				//If opdrachtgevern = Covidien, then we don't need to print out labels
				{
					// TB - add a check for shipping types that don't need a label
					if (!LIST_OF_NOT_PRINTED_SHIPPING_TYPES.Contains(Convert.ToString(dsShippingType.Tables[0].Rows[0][PmxShippingTypeTable.Columns.Name.NAME]).ToUpper()))
					{
						for (int iLoop = 1; iLoop <= numberOfColli; iLoop++)
						{
							//print label
							m_sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintLabel), luid, string.Format("{0}/{1}", iLoop, numberOfColli), session.DeviceInstanceId, reportPath, logisticCarrierCode);
						}

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", "MSG_QUESTION_REPRINT_LABEL",
							"TitleKey", "MSG_TITLE_REPRINT_LABEL"));

						msg = WaitForMessage();

						if (msg.Name.EndsWith(".Yes"))
						{
							goto Step_PrintLabel;
						}
					}
				}

				goto Step_CreateSAPDelivery;

			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}

				//retry?
				//ask user if he wants to retry
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_REPRINT_LABEL",
						"MessageKey", "MSG_QUESTION_REPRINT_LABEL"));
				msg = WaitForMessage();

				if (msg.Name.ToLower().EndsWith(".yes"))
				{
					goto Step_PrintLabel;
				}

				goto Step_CreateSAPDelivery;

			}

			#endregion

		Step_CreateSAPDelivery:
			
			#region CREATE SAP DELIVERY

			salesDeliveries = null;

			try
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
				   this.DefaultCultureInfo, BuildParamCollection(
				   "MessageKey", "MSG_ADDING_SALES_DELIVERIES"));


				Collection<int> listOfPackedPickLists = new Collection<int>() { pickListDocEntry };

				m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null, delegate(PmxDbConnection conn, object[] parameters)
				{
					PmxSalesDeliveryNoteProvider prov = new PmxSalesDeliveryNoteProvider(conn);

					salesDeliveries = prov.ShipAllItemsForPickList(listOfPackedPickLists, deliveryData, null);

					return null;
				});

				goto Step_CallSalesDeliveryCreatedHook;
			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}

				//retry?
				//ask user if he wants to retry
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_RETRY_UPDATE",
						"MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
				msg = WaitForMessage();

				if (msg.Name.ToLower().EndsWith(".yes"))
				{
					goto Step_CreateSAPDelivery;
				}

				goto Step_SelectPickList;

			}


			#endregion

		Step_CallSalesDeliveryCreatedHook:

			#region CALL SALES DELIVERY CREATED HOOK FLOW


			//sales delivery hook flow
			m_documentRefsSalesDeliveryCreatedHookFlow = salesDeliveries;

			try
			{
				//call flow
				base.StartSubFlowAndWait(m_salesDeliveryCreatedHookFlow, true);

			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();

				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}
			}

			goto Step_SelectPickList;

			#endregion


		}

		#endregion



		#region HELPERS


		/// <summary>
		/// Executes the print label.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOsePrinter ExecutePrintLabel(PmxDbConnection conn, object[] parameters)
		{
			PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

			Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
			reportParameters.Add(new ReportParameter("@luid", (int)parameters[0]));
			reportParameters.Add(new ReportParameter("@colli", (string)parameters[1]));
			reportParameters.Add(new ReportParameter("@logisticCarrier", (string)parameters[4]));

			return prov.PrintReport(reportParameters, (string)parameters[3], Definitions.NUMBER_OF_LABELS, (string)parameters[2], null, false);
		}

		/// <summary>
		/// Executes the get PMX warehouse info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxOseProvider oseProv = new PmxOseProvider(conn);
			return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
		}

		/// <summary>
		/// Gets the number of rows.
		/// </summary>
		/// <param name="ds">The ds.</param>
		/// <returns></returns>
		private int GetNumberOfRows(DataSet ds)
		{
			if (ds != null &&
				ds.Tables.Count > 0)
			{
				return ds.Tables[0].Rows.Count;
			}

			return 0;
		}

		/// <summary>
		/// Executes the get item info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
			return prov.GetCachedItemInfo((string)parameters[0]);
		}


        #endregion
    }




	#region BUILD QUERY

	/// <summary>
	/// Static class grouping our native queries
	/// </summary>
	internal static class BuildQuery
	{


		/// <summary>
		/// Checks the covidien.
		/// Added by Tom
		/// </summary>
		/// <param name="luid">The luid.</param>
		/// <returns></returns>
		public static string CheckCovidien(int luid)
		{
			return string.Format(@"SELECT  DISTINCT dbo.PMX_PLHE.PicklistType
	            FROM         dbo.PMX_PLHE INNER JOIN
	                                  dbo.PMX_PLPH ON dbo.PMX_PLHE.PickListProposalEntry = dbo.PMX_PLPH.DocEntry INNER JOIN
	                                  dbo.PMX_PLPL ON dbo.PMX_PLPH.DocEntry = dbo.PMX_PLPL.DocEntry INNER JOIN
	                                  dbo.ORDR ON dbo.PMX_PLPL.BaseEntry = dbo.ORDR.DocEntry AND dbo.PMX_PLPL.BaseType = dbo.ORDR.ObjType INNER JOIN
	                                  dbo.PMX_LPLM ON dbo.PMX_PLHE.DocEntry = dbo.PMX_LPLM.PickListDocEntry INNER JOIN
	                                  dbo.PMX_MVLI ON dbo.PMX_LPLM.MoveDocEntry = dbo.PMX_MVLI.DocEntry AND 
	                                  dbo.PMX_LPLM.MoveLineNum = dbo.PMX_MVLI.LineNum LEFT OUTER JOIN
	                                  dbo.OSHP ON dbo.ORDR.TrnspCode = dbo.OSHP.TrnspCode
	            WHERE    (dbo.PMX_MVLI.DestLogUnitIdentKey = {0}) AND dbo.ORDR.U_PMX_DR_IMSC in ( '103003','103005')", luid.ToString());


		}


		/// <summary>
		/// Gets the order information.
		/// Added by Tom
		/// </summary>
		/// <param name="luid">The luid.</param>
		/// <returns></returns>
		public static string GetOrderInfo(int luid)
		{
			return string.Format(@"SELECT  DISTINCT dbo.ORDR.U_PMX_DR_IMSC
        FROM         dbo.PMX_PLHE INNER JOIN
                              dbo.PMX_PLPH ON dbo.PMX_PLHE.PickListProposalEntry = dbo.PMX_PLPH.DocEntry INNER JOIN
                              dbo.PMX_PLPL ON dbo.PMX_PLPH.DocEntry = dbo.PMX_PLPL.DocEntry INNER JOIN
                              dbo.ORDR ON dbo.PMX_PLPL.BaseEntry = dbo.ORDR.DocEntry AND dbo.PMX_PLPL.BaseType = dbo.ORDR.ObjType INNER JOIN
                              dbo.PMX_LPLM ON dbo.PMX_PLHE.DocEntry = dbo.PMX_LPLM.PickListDocEntry INNER JOIN
                              dbo.PMX_MVLI ON dbo.PMX_LPLM.MoveDocEntry = dbo.PMX_MVLI.DocEntry AND 
                              dbo.PMX_LPLM.MoveLineNum = dbo.PMX_MVLI.LineNum LEFT OUTER JOIN
                              dbo.OSHP ON dbo.ORDR.TrnspCode = dbo.OSHP.TrnspCode
        WHERE    (dbo.PMX_MVLI.DestLogUnitIdentKey = {0}) 
		 AND dbo.ORDR.U_PMX_INFC = 'Y'", luid.ToString());


		}


		public static string GetShippingTypeForPickList(int pickListDocEntry)
		{

			//SELECT     dbo.OSHP.U_PMX_AUSH
			//FROM         dbo.PMX_PLHE INNER JOIN
			//          dbo.PMX_PLPH ON dbo.PMX_PLHE.PickListProposalEntry = dbo.PMX_PLPH.DocEntry INNER JOIN
			//          dbo.PMX_PLPL ON dbo.PMX_PLPH.DocEntry = dbo.PMX_PLPL.DocEntry INNER JOIN
			//          dbo.ORDR ON dbo.PMX_PLPL.BaseEntry = dbo.ORDR.DocEntry AND dbo.PMX_PLPL.BaseType = dbo.ORDR.ObjType LEFT OUTER JOIN
			//          dbo.OSHP ON dbo.ORDR.TrnspCode = dbo.OSHP.TrnspCode

			StringBuilder strBuilder = new StringBuilder();
			strBuilder.Append("SELECT ");

			SqlCommandHelper.AddTableColumnName(strBuilder, PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.AutomaticShipping.NAME);

			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.Name.NAME);

			strBuilder.Append(" FROM ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));

			strBuilder.Append(" INNER JOIN ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListProposalTable.NAME));
			strBuilder.Append(" ON ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalTable.NAME, PmxPickListProposalTable.Columns.DocEntry.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.PickListProposalEntry.NAME);

			strBuilder.Append(" INNER JOIN ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME));
			strBuilder.Append(" ON ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalTable.NAME, PmxPickListProposalTable.Columns.DocEntry.NAME);

			strBuilder.Append(" INNER JOIN ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(SboSalesOrderTable.NAME));
			strBuilder.Append(" ON ");
			SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocEntry.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME);
			strBuilder.Append(" AND ");
			SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.ObjType.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseType.NAME);

			strBuilder.Append(" LEFT OUTER JOIN ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxShippingTypeTable.NAME));
			strBuilder.Append(" ON ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.Code.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.TrnspCode.NAME);



			strBuilder.Append(" WHERE ");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.NAME));
			strBuilder.Append(".");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.DocEntry.NAME));
			strBuilder.Append(" = ");
			strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, CultureInfo.InvariantCulture));


			return strBuilder.ToString();
		}


		/// <summary>
		/// Gets the LUID.
		/// </summary>
		/// <param name="pickListDocEntry">The pick list doc entry.</param>
		/// <returns></returns>
		public static string GetLUID(int pickListDocEntry)
		{

			//SELECT     MAX(dbo.PMX_MVLI.DestLogUnitIdentKey) AS DestLogUnitIdentKey
			//FROM         dbo.PMX_LPLM INNER JOIN
			//                      dbo.PMX_MVLI ON dbo.PMX_LPLM.MoveDocEntry = dbo.PMX_MVLI.DocEntry AND dbo.PMX_LPLM.MoveLineNum = dbo.PMX_MVLI.LineNum
			//WHERE     (dbo.PMX_LPLM.PickListDocEntry = 279)


			StringBuilder strBuilder = new StringBuilder();
			strBuilder.Append("SELECT MAX (");

			SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);
			strBuilder.Append(") AS ");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME));

			strBuilder.Append(" FROM ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLinkPickListLastMoveTable.NAME));

			strBuilder.Append(" INNER JOIN ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMoveLineTable.NAME));
			strBuilder.Append(" ON ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DocEntry.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveDocEntry.NAME);
			strBuilder.Append(" AND ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.LineNum.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveLineNum.NAME);

			strBuilder.Append(" WHERE ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME);
			strBuilder.Append(" = ");
			strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, CultureInfo.InvariantCulture));

			return strBuilder.ToString();
		}





		/// <summary>
		/// Gets the name of the products from stock order by.
		/// </summary>
		/// <param name="gtin">The gtin.</param>
		/// <param name="itemCode">The item code.</param>
		/// <param name="packagingUnit">The packaging unit.</param>
		/// <param name="onlyLogisticCarriers">The only logistic carriers.</param>
		/// <param name="dbTool">The database tool.</param>
		/// <returns></returns>
		public static string GetProducts(string gtin, string itemCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit packagingUnit, bool? onlyLogisticCarriers, DbTool dbTool)
		{
			DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
			strBuilder.Append("SELECT DISTINCT");

			strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
			strBuilder.Append(".");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemCode.NAME));
			strBuilder.Append(" AS ");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.PRODUCT_CODE.Name));

			strBuilder.Append(", ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
			strBuilder.Append(".");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));
			strBuilder.Append(" AS ");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name));

			strBuilder.Append(", ");
			strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.NAME, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME, ProductDataSet.Columns.DEFAULT_LOCATION_CODE.Name));

			strBuilder.Append(", ");
			Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, packagingUnit, ProductDataSet.Columns.GTIN.Name);

			strBuilder.Append(", ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
			strBuilder.Append(".");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.Columns.HasBestBeforeDate.NAME));
			strBuilder.Append(" AS ");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.HAS_BEST_BEFORE.Name));

			strBuilder.Append(", ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
			strBuilder.Append(".");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ManBtchNum.NAME));
			strBuilder.Append(" AS ");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.MANBTCHNUM.Name));

			strBuilder.Append(" FROM ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));


			string operand = " WHERE ";
			if (!string.IsNullOrEmpty(gtin))
			{
				strBuilder.Append(operand);
				Produmex.Sbo.Logex.Data.SboCommands.AddWhereClauseSearchForBarcode(strBuilder, gtin, false);
				operand = " AND ";
			}

			if (!string.IsNullOrEmpty(itemCode))
			{
				strBuilder.Append(operand);
				strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemCode.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
				operand = " AND ";
			}


			if (onlyLogisticCarriers.HasValue)
			{
				strBuilder.Append(operand);
				strBuilder.Append(SqlCommandHelper.EscapeTableName(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.Columns.IsLogisticCarrier.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(onlyLogisticCarriers.Value ? SboBooleanColumnValues.Y.VALUE : SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));
				operand = " AND ";

			}

			strBuilder.Append(" ORDER BY ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
			strBuilder.Append(".");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));
			return strBuilder.ToString();
		}




		internal static string GetPickListForPacking(DbTool dbTool)
		{
			DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
			strBuilder.Append("SELECT DISTINCT ");

			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
			strBuilder.Append(" AS ");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(PickListDataSet.Columns.DOC_ENTRY.Name));

			strBuilder.Append(", CAST(");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));
			strBuilder.Append(".");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.DocEntry.NAME));
			strBuilder.Append(" AS NVARCHAR) AS ");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(PickListDataSet.Columns.DOC_ENTRY_AS_STRING.Name));

			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DueDate.NAME, PickListDataSet.Columns.DUE_DATE.Name);
			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.CardName.NAME, PickListDataSet.Columns.CUSTOMER_NAME.Name);
			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.ShipToAddress.NAME, PickListDataSet.Columns.CUSTOMER_DELIVERY_ADDRESS.Name);
			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.ShipToCode.NAME, PmxPickListTable.Columns.ShipToCode.NAME);
			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.CardCode.NAME, PickListDataSet.Columns.CUSTOMER_CODE.Name);
			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DestStorLocCode.NAME, PickListDataSet.Columns.DEST_STORLOC_CODE.Name);
			strBuilder.AppendFormat(", {0}",
				SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPickListProposalTable.NAME, PmxPickListProposalTable.Columns.ShippingID.NAME, PickListDataSet.Columns.SHIPPING_ID.Name));

			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.MoveToWarehouse.NAME, PickListDataSet.Columns.MOVE_TO_WAREHOUSE_CODE.Name);
			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, PickListDataSet.Columns.MOVE_TO_WAREHOUSE_NAME.Name);
			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.PickListStatus.NAME, PickListDataSet.Columns.PICK_LIST_STATUS.Name);


			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.Priority.NAME);
			strBuilder.Append(", ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DueDate.NAME);


			strBuilder.Append(", CAST(");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));
			strBuilder.Append(".");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.PickPackRemarks.NAME));
			strBuilder.Append(" AS NVARCHAR (4000)) AS ");
			strBuilder.Append(SqlCommandHelper.EscapeColumnName(PickListDataSet.Columns.PICK_PACK_REMARKS.Name));


			strBuilder.Append("\r\n FROM ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));


			strBuilder.Append("\r\n INNER JOIN ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMoviantoAlternatePackingViewDefinition.NAME));
			strBuilder.Append(" ON ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoviantoAlternatePackingViewDefinition.NAME, PmxMoviantoAlternatePackingViewDefinition.Columns.PickListDocEntry.NAME);

			strBuilder.Append("\r\n INNER JOIN ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWaveTable.NAME));
			strBuilder.Append(" ON ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxWaveTable.NAME, PmxWaveTable.Columns.InternalKey.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.WaveKey.NAME);

			strBuilder.Append("\r\n INNER JOIN ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME));
			strBuilder.Append(" ON ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
			strBuilder.Append(" = ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME);

			strBuilder.AppendFormat("\r\n INNER JOIN {0}",
				SqlCommandHelper.EscapeTableName(PmxPickListProposalTable.NAME));
			strBuilder.AppendFormat(" ON {0} = {1}",
				SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalTable.NAME, PmxPickListProposalTable.Columns.DocEntry.NAME),
				SqlCommandHelper.EscapeTableColumnName(PmxPickListTable.NAME, PmxPickListTable.Columns.PickListProposalEntry.NAME)
				);

			strBuilder.AppendFormat("\r\n LEFT JOIN {0}",
				SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
			strBuilder.AppendFormat(" ON {0} = {1}",
				SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME),
				SqlCommandHelper.EscapeTableColumnName(PmxPickListTable.NAME, PmxPickListTable.Columns.MoveToWarehouse.NAME));



			strBuilder.Append("\r\n  ORDER BY ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.Priority.NAME);
			strBuilder.Append("\r\n , ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DueDate.NAME);
			strBuilder.Append("\r\n , ");
			SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);


			return strBuilder.ToString();
		}



	}


	#endregion

}
