﻿using System;
using System.Diagnostics;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using System.Xml;
using System.Web;
using System.Net;
using System.IO;
using System.Threading;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DataSetClient;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;      
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Sbo.Logex.MoviantoNL.Providers;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables;



// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
	/// <summary>
	/// Cycle counting
	/// </summary>
	public class WorkflowScript_PackagingMoviantoScript : WorkflowInstanceScriptBase
	{
        private static class Definitions
        {
            public const string REPORT_COLLIPACKAGELABEL = @"MoviantoColliPackageLabel.rpt";

            public const string COMPANY_CODE  = @"151";
            public const string USER_ID       = @"Upload@moviantobenelux.com";
            public const string PASSWORD      = @"UploadMoviantoSAP";

            // DEMO
            public const string REFERENCE     = @"TSTPETER4";
        }

		// Input parameters
		public ReadOnlyBinder<CultureInfo>   DefaultCultureInfo;
		public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		private ISboProviderService m_sboProviderService;
       
        private class ColliData 
        {
            public string ColliID;

            public double Width;
            public double Length;
            public double Height;
            public double Weight;

            public ColliData(string pColliID, double pWidth, double pLength, double pHeight, double pWeight)
            {
                ColliID = pColliID;
                Width   = pWidth;
                Length  = pLength;
                Height  = pHeight;
                Weight  = pWeight;
            }
        };

		#region Constructors

		public WorkflowScript_PackagingMoviantoScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory): base(parent, factory)
		{
		}

		#endregion

		#region WorkflowInstanceScriptBase Members

		/// <summary>
		/// Executes this instance.
		/// </summary>
		protected override void Execute()
		{
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
			ProdumexError pmxError = null;

			string initialErrorKey = null;
			string deviceID = session.DeviceInstanceId;
			SboUserInfo userInfo = m_sboProviderService.GetCurrentUser();
			string query = null;
            DataSet dsPrincipalLayout = null;



            Message msg                = null;


            int     pickListDocEntry   = 0;
            int     salesorderDocEntry = 0;
            string  location           = String.Empty;

            string  colliID            = String.Empty;
            double  width              = 0;
            double  length             = 0;
            double  height             = 0;
            double  weight             = 0;

            List<ColliData> Collis = new List<ColliData>();

            string Carrier = String.Empty;
            string AWBNr   = String.Empty;
            string Rate    = String.Empty;

            DataSet dsAddress    = null;
            DataSet dsSalesOrder = null;

            string ContactName = String.Empty;
            string ContactTel  = String.Empty;

            PmxMovPLCD lColliExists      = null;
            Boolean    lLocalColliExists = false;
            string DeliveryLayout = null;


        Step_SelectPickList:
          

            pickListDocEntry   = 0;
            salesorderDocEntry = 0;
            Collis.Clear();
            msg = null;

            query = BuildQuery.GetPickList();
            DataSet dsPickList = m_sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Picking.ISelectPickListScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection( "TitleKey"       , "Select Picklist",
                                                                               "PickListDS"     , dsPickList,
                                                                               "InitialErrorKey", null ));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".PickListSelected"))
            {
                pickListDocEntry = ExtractParameter<int>(msg.Parameters, "pickListDocEntry");
                
                DataRow[] rows = dsPickList.Tables[0].Select(string.Format("{0} = {1}", PickListDataSet.Columns.DOC_ENTRY.Name, pickListDocEntry));

                if (rows.Length > 0)
                {
                    ContactName = rows[0].IsNull("ContactName") ? String.Empty : Convert.ToString(rows[0]["ContactName"]);
                    ContactTel  = rows[0].IsNull("ContactTel") ? String.Empty : Convert.ToString(rows[0]["ContactTel"]);

                    salesorderDocEntry = Convert.ToInt32(rows[0]["SalesOrderDocEntry"]);
                    location           = Convert.ToString(rows[0]["DestStorLocCode"]);

                    dsAddress = m_sboProviderService.RunView(false, null, null, BuildQuery.GetCustomerAddress(salesorderDocEntry));

                    if (dsAddress.Tables[0].Rows.Count == 0)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                           DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", null,
                                                                                          "MessageKey", "Geen lever-adres beschikbaar",
                                                                                          "ShowButton", true));
                        msg = WaitForMessage();

                        goto Step_SelectPickList;
                    }

                    dsSalesOrder = m_sboProviderService.RunView(false, null, null, BuildQuery.GetSalesOrderInfo(salesorderDocEntry));

                    if (dsSalesOrder.Tables[0].Rows.Count == 0)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                           DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", null,
                                                                                          "MessageKey", "Geen Salesorder data beschikbaar",
                                                                                          "ShowButton", true));
                        msg = WaitForMessage();

                        goto Step_SelectPickList;
                    }
                }
                else
                {
                    goto Step_End;
                }

            }
            else if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectPickList;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_End;
            }

        Step_ScanColliID:

            lColliExists = null;
            lLocalColliExists = false;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyBarcodeScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection( "TitleKey"       , "Scan Colli ID",
                                                                               "InitialErrorKey", null,
                                                                               "AllowStopButton", (Collis.Count > 0)));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".DataEntered"))
            {
                colliID = ExtractParameter<string>(msg.Parameters, "data");


                foreach (ColliData lColliData in Collis)
                {
                    if (lColliData.ColliID.ToUpper() == colliID.ToUpper())
                    {
                        lLocalColliExists = true;
                        break;
                    }
                }
                
                m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, delegate(PmxDbConnection conn, object[] parameters)
                {
                    PmxMovPLCDProvider prov = new PmxMovPLCDProvider(conn);

                    List<PmxMovPLCD> coll = new List<PmxMovPLCD>();
                    prov.GetBOsByWhereClause(s_log, BuildQuery.GetPMX_MOV_PLCDWhereClause(colliID, conn), null, 1, false, coll, false);

                    if (coll.Count == 1)
                      lColliExists = coll[0];

                    return null;
                });


                if (lColliExists != null)
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                                       DefaultCultureInfo.Get(), BuildParamCollection("TitleKey", "Colli bestaat reeds",
                                                                                      "MessageKey", "Colli reeds gescand, wilt u de afmetingen terug ingeven?",
                                                                                      "ButtonKeyYes", "Ja",
                                                                                      "ButtonKeyNo", "Nee",
                                                                                       "AllowToGoBack", false));

                    msg = WaitForMessage();
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_GetWidth;
                    }
                    else
                    {
                        goto Step_ScanColliID;
                    }
                }
                else
                {
                    if (lLocalColliExists)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection("TitleKey", "Colli bestaat reeds",
                                                                       "MessageKey", "Colli reeds gescand in huidige picklist, wilt u de afmetingen terug ingeven?",
                                                                       "ButtonKeyYes", "Ja",
                                                                       "ButtonKeyNo", "Nee",
                                                                        "AllowToGoBack", false));

                        msg = WaitForMessage();
                        initialErrorKey = null;

                        if (msg.Name.EndsWith(".Yes"))
                        {
                            goto Step_GetWidth;
                        }
                        else
                        {
                            goto Step_ScanColliID;
                        }

                    }
                    else
                    {
                        goto Step_GetWidth;
                    }
                }

            }
            else if (msg.Name.EndsWith(".StopRequested"))
            {
                goto Step_CallWebService;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_SelectPickList;
            }


        Step_GetWidth:

            /*
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", initialErrorKey,
                                                                              "TitleKey", "MSG_TITLE_ENTER_WIDTH_IN_CM",
                                                                              "InitialQuantity", 0,
                                                                              "NumberOfDigitsAfterDecimal", 1,
                                                                              "MaximumQuantity", -1,
                                                                              "UOM", "cm"));
            */
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", initialErrorKey,
                                                                               "TitleKey", "MSG_TITLE_ENTER_WIDTH_IN_CM",
                                                                               "DefaultValue", "0",
                                                                               "AllowMultiLine", false,
                                                                               "AllowToGoBack", true,
                                                                               "MinimumNumberOfCharacters", 1));

            msg = WaitForMessage();

            if (msg.Name.EndsWith(".BackRequested"))
                goto Step_ScanColliID;

            if (msg.Name.EndsWith(".StringEntered") || msg.Name.EndsWith(".QuantityEntered"))
            {
                if (msg.Name.EndsWith(".StringEntered"))
                {
                    string lEnteredValue = ExtractParameter<string>(msg.Parameters, "stringValue");
                    lEnteredValue = lEnteredValue.Replace(",", ".").Trim();
                    while (lEnteredValue.Contains(" "))
                        lEnteredValue = lEnteredValue.Replace(" ", "");
                    Boolean lConvertSuccess = double.TryParse(lEnteredValue, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture, out width);

                    if (!lConvertSuccess)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                           DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", null,
                                                                                          "MessageKey", String.Format("Kan waarde ({0}) niet omzetten naar een nummer", lEnteredValue),
                                                                                          "ShowButton", true));
                        msg = WaitForMessage();

                        goto Step_GetWidth;
                    }
                }
                else
                    width = ExtractParameter<double>(msg.Parameters, "quantity");                
            }

        Step_GetLength:

            /*
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", initialErrorKey,
                                                                              "TitleKey", "MSG_TITLE_ENTER_LENGTH_IN_CM",
                                                                              "InitialQuantity", 0,
                                                                              "NumberOfDigitsAfterDecimal", 1,
                                                                              "MaximumQuantity", -1,
                                                                              "UOM", "cm"));
            */
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", initialErrorKey,
                                                                               "TitleKey", "MSG_TITLE_ENTER_LENGTH_IN_CM",
                                                                               "DefaultValue", "0",
                                                                               "AllowMultiLine", false,
                                                                               "AllowToGoBack", true,
                                                                               "MinimumNumberOfCharacters", 1));

            msg = WaitForMessage();

            if (msg.Name.EndsWith(".BackRequested"))
                goto Step_GetWidth;

            if (msg.Name.EndsWith(".StringEntered") || msg.Name.EndsWith(".QuantityEntered"))
            {
                if (msg.Name.EndsWith(".StringEntered"))
                {
                    string lEnteredValue = ExtractParameter<string>(msg.Parameters, "stringValue");
                    lEnteredValue = lEnteredValue.Replace(",", ".").Trim();
                    while (lEnteredValue.Contains(" "))
                        lEnteredValue = lEnteredValue.Replace(" ", "");
                    Boolean lConvertSuccess = double.TryParse(lEnteredValue, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture, out length);

                    if (!lConvertSuccess)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                           DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", null,
                                                                                          "MessageKey", String.Format("Kan waarde ({0}) niet omzetten naar een nummer", lEnteredValue),
                                                                                          "ShowButton", true));
                        msg = WaitForMessage();

                        goto Step_GetLength;
                    }
                }
                else
                    length = ExtractParameter<double>(msg.Parameters, "quantity");                
            }

        Step_GetHeight:

            /*
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", initialErrorKey,
                                                                               "TitleKey", "MSG_TITLE_ENTER_HEIGHT_IN_CM",
                                                                               "InitialQuantity", 0,
                                                                               "NumberOfDigitsAfterDecimal", 1,
                                                                               "MaximumQuantity", -1,
                                                                               "UOM", "cm"));
            */
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", initialErrorKey,
                                                                               "TitleKey", "MSG_TITLE_ENTER_HEIGHT_IN_CM",
                                                                               "DefaultValue", "0",
                                                                               "AllowMultiLine", false,
                                                                               "AllowToGoBack", true,
                                                                               "MinimumNumberOfCharacters", 1));

            msg = WaitForMessage();

            if (msg.Name.EndsWith(".BackRequested"))
                goto Step_GetLength;

            if (msg.Name.EndsWith(".StringEntered") || msg.Name.EndsWith(".QuantityEntered"))
            {
                if (msg.Name.EndsWith(".StringEntered"))
                {
                    string lEnteredValue = ExtractParameter<string>(msg.Parameters, "stringValue");
                    lEnteredValue = lEnteredValue.Replace(",", ".").Trim();
                    while (lEnteredValue.Contains(" "))
                        lEnteredValue = lEnteredValue.Replace(" ", "");
                    Boolean lConvertSuccess = double.TryParse(lEnteredValue, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture, out height);

                    if (!lConvertSuccess)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                           DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", null,
                                                                                          "MessageKey", String.Format("Kan waarde ({0}) niet omzetten naar een nummer", lEnteredValue),
                                                                                          "ShowButton", true));
                        msg = WaitForMessage();

                        goto Step_GetHeight;
                    }
                }
                else
                    height = ExtractParameter<double>(msg.Parameters, "quantity");                
            }

        Step_GetWeight:

            /*
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", initialErrorKey,
                                                                               "TitleKey", "MSG_TITLE_ENTER_WEIGHT_IN_KG",
                                                                               "InitialQuantity", 0,
                                                                               "NumberOfDigitsAfterDecimal", 2,
                                                                               "MaximumQuantity", -1,
                                                                               "UOM", "kg"));
            */
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", initialErrorKey,
                                                                               "TitleKey", "MSG_TITLE_ENTER_WEIGHT_IN_KG",
                                                                               "DefaultValue", "0",
                                                                               "AllowMultiLine", false,
                                                                               "AllowToGoBack", true,
                                                                               "MinimumNumberOfCharacters", 1));


            msg = WaitForMessage();

            if (msg.Name.EndsWith(".BackRequested"))
                goto Step_GetHeight;

            if (msg.Name.EndsWith(".StringEntered") || msg.Name.EndsWith(".QuantityEntered"))
            {
                if (msg.Name.EndsWith(".StringEntered"))
                {
                    string lEnteredValue = ExtractParameter<string>(msg.Parameters, "stringValue");
                    lEnteredValue = lEnteredValue.Replace(",", ".").Trim();
                    while (lEnteredValue.Contains(" "))
                        lEnteredValue = lEnteredValue.Replace(" ", "");
                    Boolean lConvertSuccess = double.TryParse(lEnteredValue, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture, out weight);

                    if (!lConvertSuccess)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                           DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", null,
                                                                                          "MessageKey", String.Format("Kan waarde ({0}) niet omzetten naar een nummer", lEnteredValue),
                                                                                          "ShowButton", true));
                        msg = WaitForMessage();

                        goto Step_GetWeight;
                    }
                }
                else
                    weight = ExtractParameter<double>(msg.Parameters, "quantity");


                if (lColliExists != null)
                {
                    lColliExists.Width = width;
                    lColliExists.Length = length;
                    lColliExists.Height = height;
                    lColliExists.Weight = weight;

                    m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, delegate(PmxDbConnection conn, object[] parameters)
                    {
                        PmxMovPLCDProvider prov = new PmxMovPLCDProvider(conn);
                        prov.UpdateBO(lColliExists, false, false);
                        return null;
                    });
                }
                else
                {
                    Boolean lColliAdded = false;

                    foreach (ColliData lColliData in Collis)
                    {
                        if (lColliData.ColliID.ToUpper() == colliID.ToUpper())
                        {
                            lColliData.Width  = width;
                            lColliData.Length = length;
                            lColliData.Height = height;
                            lColliData.Weight = weight;

                            lColliAdded = true;
                            break;
                        }
                    }

                    if (!lColliAdded)
                        Collis.Add(new ColliData(colliID, width, length, height, weight));
                }

                goto Step_ScanColliID;
            }

        Step_CallWebService:

            Carrier = String.Empty;
            AWBNr   = String.Empty;
            Rate    = String.Empty;

            string lXMLRequest = BuildXMLRequest(Collis, salesorderDocEntry.ToString(), ContactName, ContactTel, dsPickList, dsAddress, dsSalesOrder);
            // DEMO DATA
            //string lXMLRequest = BuildXMLRequest(Collis, "TESTPETER", "test customer", "Street 1", "", "Berlin", "", "DE", "10115", "Abc", "115-7733", dsPickList, dsAddress, dsSalesOrder);
            string lXMLReply   = postXMLData("http://www2.shipitsmarter.com/movianto/insert_shipment.asp", lXMLRequest);

            if (lXMLReply != String.Empty)
            {
				string lFirstPart = null;
                try
                {
                  XmlDocument xml = new XmlDocument();
                  xml.LoadXml(lXMLReply);

                  XmlNode lStatus = xml.SelectSingleNode("response/status");

                  if (lStatus == null || lStatus.InnerText != "OK")
                      throw new Exception("Response not valid");

                  XmlNode lUrl = xml.SelectSingleNode("response/url");

                  if (lUrl != null)
                  {
                      ProcessStartInfo startInfo = new ProcessStartInfo("IExplore.exe");
                      startInfo.WindowStyle = ProcessWindowStyle.Normal;                     
                      startInfo.Arguments = lUrl.InnerText;
                      Process.Start(startInfo);

                      session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                      DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", null,
                                                                     "MessageKey", "Ga door als label geprint is.",
                                                                     "ShowButton", true));
                      msg = WaitForMessage();
                  }

                  string lXMLRequestStatus = BuildXMLStatus(salesorderDocEntry.ToString());
                  string lXMLReplyStatus   = postXMLData("http://www2.shipitsmarter.com/movianto/status_shipment.asp", lXMLRequestStatus);

                  XmlDocument xmlStatus = new XmlDocument();
                  lFirstPart = lXMLReplyStatus.Substring(0, lXMLReplyStatus.IndexOf(@"<stripStatus>"));
                  xmlStatus.LoadXml(lFirstPart);                   

                  XmlNode lNodeRate = xmlStatus.SelectSingleNode("despatchConfirmationAWB/despatchHeader/rate");
                  if (lNodeRate != null)
                    Rate = lNodeRate.InnerText;

                  Carrier = xmlStatus.SelectSingleNode("despatchConfirmationAWB/despatchHeader/shippedBy").InnerText;
                  AWBNr   = xmlStatus.SelectSingleNode("despatchConfirmationAWB/despatchHeader/trackingNumber").InnerText;  
                }
                catch (Exception ex)
                {
					s_log.Error("", ex);
					s_log.ErrorFormat("lXMLRequest = {0}", lXMLRequest);
					s_log.ErrorFormat("lXMLReply = {0}", lXMLReply);
					s_log.ErrorFormat("lFirstPart = {0}", lFirstPart);

                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                       DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", null,
                                                                                      "MessageKey", "Fout bij ophalen SIS gegevens",
                                                                                      "ShowButton", true));
                    msg = WaitForMessage();
                }
                finally 
                {
                }

                m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, delegate(PmxDbConnection conn, object[] parameters)
                 { 
                   PmxMovPLCDProvider prov = new PmxMovPLCDProvider(conn);                   

                   foreach (ColliData colli in Collis)
                   {
                     using (PmxMovPLCD obj = prov.GetNewBO())
                     {
                       obj.PickListDocEntry   = pickListDocEntry;
                       obj.SalesOrderDocEntry = salesorderDocEntry;
                       obj.ColliID            = colli.ColliID;
                       obj.Width              = colli.Width;
                       obj.Length             = colli.Length;
                       obj.Height             = colli.Height;
                       obj.Weight             = colli.Weight;
                       obj.Carrier            = Carrier;
                       obj.AWBNr              = AWBNr;

                       if (!String.IsNullOrEmpty(Rate))
                           obj.Rate = Convert.ToDouble(Rate, CultureInfo.InvariantCulture);

                       prov.AddBO(obj);
                     }
                   }

                   return null;
                 });

                goto Step_PrintReport;
             }                        

        Step_PrintReport:
           
                try
                {
                //Definitions.REPORT_COLLIPACKAGELABEL = BuildQuery.GetPrincipalLayout(salesorderDocEntry);
                query = BuildQuery.GetPrincipalLayout(salesorderDocEntry);
                dsPrincipalLayout = m_sboProviderService.RunView(false, null, null, query);
                DeliveryLayout = dsPrincipalLayout.Tables[0].Rows[0]["Name"].ToString();
        
                    //print report
                m_sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintColliPackageLabelReport), pickListDocEntry, salesorderDocEntry, session.DeviceInstanceId, DeliveryLayout, location);

                    Thread.Sleep(750);

                }
                catch (Exception ex)
                {
                    s_log.Error(ex.ToString());
                    throw ex;
                }

		Step_SendDSP:

			using (PmxDbConnection conn = PmxDbConnectionMgr.GI.GetOpenDirectConnection())
			{
				PmxgeneralProviderMoviantoNL prov = new PmxgeneralProviderMoviantoNL(conn);

				bool canExport = false;
				query = BuildQuery.CheckExportDSPNeeded(pickListDocEntry, conn.DbCultureInfo);
				using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, conn))
				{
					canExport = !rs.EoF;
				}
				if (canExport)
				{

					prov.ExportDSPFile(pickListDocEntry, null, conn);
				}

			}
                
			goto Step_SelectPickList;
            

        Step_End:

            pickListDocEntry = 0;
		}

		#endregion



		#region HELPERS

		/// <summary>
		/// Executes the get PMX warehouse info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxOseProvider oseProv = new PmxOseProvider(conn);
			return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
		}

		/// <summary>
		/// Gets the number of rows.
		/// </summary>
		/// <param name="ds">The ds.</param>
		/// <returns></returns>
		private int GetNumberOfRows(DataSet ds)
		{
			if (ds != null &&
				ds.Tables.Count > 0)
			{
				return ds.Tables[0].Rows.Count;
			}

			return 0;
		}

		/// <summary>
		/// Executes the get item info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
			return prov.GetCachedItemInfo((string)parameters[0]);
		}

        /// <summary>
        /// Executes the report.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintColliPackageLabelReport(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@PickListDocEntry", (int)parameters[0]));
            reportParameters.Add(new ReportParameter("@SalesOrderDocEntry", (int)parameters[1]));
            
            return prov.PrintReport(reportParameters, (string)parameters[3], 1, (string)parameters[2], (string)parameters[4], false);
        }

        /// <summary>
        /// Executes the report.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static object AddData(PmxDbConnection conn, object[] parameters)
        {
            PmxLogisticUnitIDProvider prov = new PmxLogisticUnitIDProvider(conn);

            using (PmxLogisticUnitID luid = prov.GetNewBO())
            {
                luid.IsFullPallet = true;

                prov.AddBO(luid);
 
            }


            return null;
        }

        /// <summary>
        /// Send the xml to the specified webservice.
        /// </summary>
        /// <param name="destinationUrl">The address of the webservice</param>
        /// <param name="requestXML">The xml that should be sent.</param>
        /// <returns></returns>
        public string postXMLData(string destinationUrl, string requestXml)
        {
            /*
            // Create a request using a URL that can receive a post. 
            WebRequest request = WebRequest.Create(destinationUrl);
            // Set the Method property of the request to POST.
            request.Method = "POST";
            // Create POST data and convert it to a byte array.

            string postData = String.Format("data_xml={0}", requestXml);
            byte[] byteArray = Encoding.UTF8.GetBytes(postData);
            // Set the ContentType property of the WebRequest.
            request.ContentType = "application/x-www-form-urlencoded";
            // Set the ContentLength property of the WebRequest.
            request.ContentLength = byteArray.Length;
            // Get the request stream.
            Stream dataStream = request.GetRequestStream();
            // Write the data to the request stream.
            dataStream.Write(byteArray, 0, byteArray.Length);
            // Close the Stream object.
            dataStream.Close();
            // Get the response.
            WebResponse response = request.GetResponse();
            // Display the status.
            Console.WriteLine(((HttpWebResponse)response).StatusDescription);
            // Get the stream containing content returned by the server.
            dataStream = response.GetResponseStream();
            // Open the stream using a StreamReader for easy access.
            StreamReader reader = new StreamReader(dataStream);
            // Read the content.
            string responseFromServer = reader.ReadToEnd();
            // Display the content.
            Console.WriteLine(responseFromServer);
            // Clean up the streams.
            reader.Close();
            dataStream.Close();
            response.Close();
            */


            string resultString = String.Empty;

            using (var client = new WebClient())
            {
                //client.Headers[HttpRequestHeader.ContentType] = "application/x-www-form-urlencoded";
                //client.Encoding = Encoding.UTF8;

                var values = new NameValueCollection
                {                    
                    { "data_xml", requestXml }
                };

                var result = client.UploadValues(destinationUrl, "POST", values);
                resultString = client.Encoding.GetString(result);
            }

            return resultString;
        }

        /// <summary>
        /// Builds the XML request for the webservice to get the shipment status.
        /// </summary>        
        /// <returns></returns>        
        private static string BuildXMLStatus(string pReference)
        {
            XmlDocument xmlDoc = new XmlDocument();

            XmlDeclaration declaration = xmlDoc.CreateXmlDeclaration("1.0", "UTF-8", null);
            xmlDoc.AppendChild(declaration);

            XmlElement rootElm = xmlDoc.CreateElement("shipment_status");

            XmlAttribute attr = xmlDoc.CreateAttribute("code_company");
            attr.Value = Definitions.COMPANY_CODE;
            rootElm.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("userid");
            attr.Value = Definitions.USER_ID;
            rootElm.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("password");
            attr.Value = Definitions.PASSWORD;
            rootElm.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("version");
            attr.Value = "1.0";
            rootElm.Attributes.Append(attr);

            xmlDoc.AppendChild(rootElm);

            XmlElement dataElm = xmlDoc.CreateElement("reference");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(pReference));

            return xmlDoc.InnerXml;

            Encoding Utf8 = new UTF8Encoding(false);

            System.Xml.XmlWriterSettings settings = new System.Xml.XmlWriterSettings();
            settings.Encoding = Encoding.UTF8;  // Utf8; //  Encoding.UTF8; //  Utf8; //Encoding.UTF8;//.GetEncoding(1252);  // This is "Windows-1252"
            settings.Indent = true;
            settings.IndentChars = "    ";
            settings.NewLineChars = "\r\n";



            StringBuilder strBuilderXML = new StringBuilder();

            using (System.Xml.XmlWriter xmlWriter = System.Xml.XmlWriter.Create(strBuilderXML, settings))
            {
                xmlDoc.Save(xmlWriter);
            }

            return strBuilderXML.ToString();
        }



		private static string EscapeXMLValue(string value)
		{
			//if (!string.IsNullOrEmpty(value))
			//{
			//    return value.Replace("&", "&amp;");
			//}
			return value;
		}

        /// <summary>
        /// Builds the XML request for the webservice.
        /// </summary>        
        /// <returns></returns>        
        private static string BuildXMLRequest(List<ColliData> pColliData, string pOrderNumber, string pContactName, string pContactTel, DataSet pdsPicklist, DataSet pdsAddress, DataSet pdsSalesOrder) 
        {
            string ShippingStreet     = EscapeXMLValue(pdsAddress.Tables[0].Rows[0].IsNull(SboSalesOrderTaxExtensionTable.Columns.StreetS.NAME)  ? String.Empty : Convert.ToString(pdsAddress.Tables[0].Rows[0][SboSalesOrderTaxExtensionTable.Columns.StreetS.NAME]));
            string ShippingCity       = EscapeXMLValue(pdsAddress.Tables[0].Rows[0].IsNull(SboSalesOrderTaxExtensionTable.Columns.CityS.NAME)    ? String.Empty : Convert.ToString(pdsAddress.Tables[0].Rows[0][SboSalesOrderTaxExtensionTable.Columns.CityS.NAME]));
            string ShippingState      = EscapeXMLValue(pdsAddress.Tables[0].Rows[0].IsNull(SboSalesOrderTaxExtensionTable.Columns.StateS.NAME)   ? String.Empty : Convert.ToString(pdsAddress.Tables[0].Rows[0][SboSalesOrderTaxExtensionTable.Columns.StateS.NAME]));
            string ShippingCountry    = EscapeXMLValue(pdsAddress.Tables[0].Rows[0].IsNull(SboSalesOrderTaxExtensionTable.Columns.CountryS.NAME) ? String.Empty : Convert.ToString(pdsAddress.Tables[0].Rows[0][SboSalesOrderTaxExtensionTable.Columns.CountryS.NAME]));
            string ShippingPostalCode = EscapeXMLValue(pdsAddress.Tables[0].Rows[0].IsNull(SboSalesOrderTaxExtensionTable.Columns.ZipCodeS.NAME) ? String.Empty : Convert.ToString(pdsAddress.Tables[0].Rows[0][SboSalesOrderTaxExtensionTable.Columns.ZipCodeS.NAME]));

            string service_level      = EscapeXMLValue(pdsSalesOrder.Tables[0].Rows[0].IsNull(SboShippingTypeTable.Columns.Name.NAME) ? String.Empty : Convert.ToString(pdsSalesOrder.Tables[0].Rows[0][SboShippingTypeTable.Columns.Name.NAME]));
            string agent_building     = EscapeXMLValue(pdsSalesOrder.Tables[0].Rows[0].IsNull(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.InitiatorCode.NAME) ? String.Empty : Convert.ToString(pdsSalesOrder.Tables[0].Rows[0][Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.InitiatorCode.NAME]));
            string delivery_condition = EscapeXMLValue(pdsSalesOrder.Tables[0].Rows[0].IsNull(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField8.NAME) ? String.Empty : Convert.ToString(pdsSalesOrder.Tables[0].Rows[0][Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField8.NAME]));
            string text07             = EscapeXMLValue(pdsSalesOrder.Tables[0].Rows[0].IsNull(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField7.NAME) ? String.Empty : Convert.ToString(pdsSalesOrder.Tables[0].Rows[0][Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField7.NAME]));
            string text06             = EscapeXMLValue(pdsSalesOrder.Tables[0].Rows[0].IsNull(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField6.NAME) ? String.Empty : Convert.ToString(pdsSalesOrder.Tables[0].Rows[0][Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField6.NAME]));
            string NumAtCard          = EscapeXMLValue(pdsSalesOrder.Tables[0].Rows[0].IsNull(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.NumAtCard.NAME) ? String.Empty : Convert.ToString(pdsSalesOrder.Tables[0].Rows[0][Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.NumAtCard.NAME]));
            string address11          = EscapeXMLValue(pdsSalesOrder.Tables[0].Rows[0].IsNull(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.CardName.NAME) ? String.Empty : Convert.ToString(pdsSalesOrder.Tables[0].Rows[0][Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.CardName.NAME]));
            string address13          = EscapeXMLValue(pdsSalesOrder.Tables[0].Rows[0].IsNull(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.Address2.NAME) ? String.Empty : Convert.ToString(pdsSalesOrder.Tables[0].Rows[0][Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.Address2.NAME]));

            string account            = "000001";
            

            // DEMO
            // agent_building = "103005";
            // account        = "00001";
            // service_level  = "EXPRESS";


            double lTotalWeight = 0;
            foreach (ColliData lColli in pColliData)
                lTotalWeight += lColli.Weight;

            XmlDocument xmlDoc = new XmlDocument();

            XmlDeclaration declaration = xmlDoc.CreateXmlDeclaration("1.0", "UTF-8", null);
            xmlDoc.AppendChild(declaration);

            XmlElement rootElm = xmlDoc.CreateElement("insert_shipment");

            XmlAttribute attr = xmlDoc.CreateAttribute("code_company");
            attr.Value = Definitions.COMPANY_CODE;
            rootElm.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("userid");
            attr.Value = Definitions.USER_ID;
            rootElm.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("password");
            attr.Value = Definitions.PASSWORD;
            rootElm.Attributes.Append(attr);

            attr = xmlDoc.CreateAttribute("agent_building");
            attr.Value = agent_building;
            rootElm.Attributes.Append(attr);

            xmlDoc.AppendChild(rootElm);


            //ACTION
            XmlElement dataElm = xmlDoc.CreateElement("action");
            rootElm.AppendChild(dataElm);
            //dataElm.AppendChild(xmlDoc.CreateTextNode("STORE_AWB"));
            // OUTPUT_TYPE
            dataElm = xmlDoc.CreateElement("output_type");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode("XML"));
            // REFERENCE
            dataElm = xmlDoc.CreateElement("reference");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(pOrderNumber));
            //ACCOUNT
            dataElm = xmlDoc.CreateElement("account");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(account));
            //ADDRESS11
            dataElm = xmlDoc.CreateElement("address11");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(address11));
            //ADDRESS12
            dataElm = xmlDoc.CreateElement("address12");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(ShippingStreet));
            //ADDRESS13
            dataElm = xmlDoc.CreateElement("address13");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(address13));
            //ADDRESS14
            dataElm = xmlDoc.CreateElement("address14");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(ShippingCity));
            //CODE_STATE_TO
            dataElm = xmlDoc.CreateElement("code_state_to");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(ShippingState));
            //CODE_COUNTRY_TO
            dataElm = xmlDoc.CreateElement("code_country_to");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(ShippingCountry));
            //POSTCODE_DELIVERY
            dataElm = xmlDoc.CreateElement("postcode_delivery");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(ShippingPostalCode));
            //DELIVER
            dataElm = xmlDoc.CreateElement("deliver");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(pContactName));
            //DELIVER_FAX
            dataElm = xmlDoc.CreateElement("deliver_fax");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(pContactTel));
            //DELIVERY
            dataElm = xmlDoc.CreateElement("delivery");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(String.Empty));
            //SERVICE_LEVEL
            dataElm = xmlDoc.CreateElement("service_level");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(service_level));
            //DELIVERY_CONDITION
            dataElm = xmlDoc.CreateElement("delivery_condition");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(delivery_condition));
            //WEIGHT
            dataElm = xmlDoc.CreateElement("weight");
            rootElm.AppendChild(dataElm);
            dataElm.AppendChild(xmlDoc.CreateTextNode(SqlCommandHelper.FormatAndEscapeColumnValue(lTotalWeight, CultureInfo.InvariantCulture)));
            
            //REFERENCE_LIST
            XmlElement referenceList = xmlDoc.CreateElement("reference_list");
            rootElm.AppendChild(referenceList);
            {
                dataElm = xmlDoc.CreateElement("reference_tag");
                referenceList.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode("lieferschein"));

                dataElm = xmlDoc.CreateElement("reference_value");
                referenceList.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode(String.Format("{0}-{1}", text07, text06)));

                dataElm = xmlDoc.CreateElement("reference_tag");
                referenceList.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode("customer_po_nr"));

                dataElm = xmlDoc.CreateElement("reference_value");
                referenceList.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode(NumAtCard));
            }

            //SHIPMENT_LINE
            int lLineID = 0;
            foreach (ColliData lColli in pColliData)
            {
                lLineID++;

                XmlElement shipmentline = xmlDoc.CreateElement("shipment_line");

                attr = xmlDoc.CreateAttribute("id");
                attr.Value = lLineID.ToString();
                shipmentline.Attributes.Append(attr);

                rootElm.AppendChild(shipmentline);

                //PACKAGE
                dataElm = xmlDoc.CreateElement("package");
                shipmentline.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode("COLLI"));
                //DESCRIPTION
                dataElm = xmlDoc.CreateElement("description");
                shipmentline.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode(String.Empty));
                //COLLO_NUMBER
                dataElm = xmlDoc.CreateElement("collo_number");
                shipmentline.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode(lColli.ColliID));
                //QUANTITY
                dataElm = xmlDoc.CreateElement("quantity");
                shipmentline.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode("1"));
                //WEIGHT
                dataElm = xmlDoc.CreateElement("weight");
                shipmentline.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode(SqlCommandHelper.FormatAndEscapeColumnValue(lColli.Weight, CultureInfo.InvariantCulture)));
                //WIDTH
                dataElm = xmlDoc.CreateElement("width");
                shipmentline.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode(SqlCommandHelper.FormatAndEscapeColumnValue(lColli.Width, CultureInfo.InvariantCulture)));
                //LENGTH
                dataElm = xmlDoc.CreateElement("length");
                shipmentline.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode(SqlCommandHelper.FormatAndEscapeColumnValue(lColli.Length, CultureInfo.InvariantCulture)));
                //HEIGHT
                dataElm = xmlDoc.CreateElement("height");
                shipmentline.AppendChild(dataElm);
                dataElm.AppendChild(xmlDoc.CreateTextNode(SqlCommandHelper.FormatAndEscapeColumnValue(lColli.Height, CultureInfo.InvariantCulture)));
            }
       
            Encoding Utf8 = new UTF8Encoding(false);

            System.Xml.XmlWriterSettings settings = new System.Xml.XmlWriterSettings();
            settings.Encoding     = Encoding.UTF8;  // Utf8; //  Encoding.UTF8; //  Utf8; //Encoding.UTF8;//.GetEncoding(1252);  // This is "Windows-1252"
            settings.Indent       = true;
            settings.IndentChars  = "    ";
            settings.NewLineChars = "\r\n";
           
            StringBuilder strBuilderXML = new StringBuilder();
            using (System.Xml.XmlWriter xmlWriter = System.Xml.XmlWriter.Create(strBuilderXML, settings))
            { 
                xmlDoc.Save(xmlWriter);
            }

            // return xmlDoc.InnerXml;
            return strBuilderXML.ToString();
        }

        #endregion
    }




	#region BUILD QUERY

	/// <summary>
	/// Static class grouping our native queries
	/// </summary>
	internal static class BuildQuery
	{


		/// <summary>
		/// Checks the export DSP needed.
		/// </summary>
		/// <param name="docEntry">The document entry.</param>
		/// <param name="dbCulture">The database culture.</param>
		/// <returns></returns>
		public static string CheckExportDSPNeeded(int docEntry, CultureInfo dbCulture)
		{
			string tableNameInitiator = "INITIATOR";

			StringBuilder query = new StringBuilder(255);
			query.Append("SELECT ");
			SqlCommandHelper.AddTableColumnName(query, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);


			query.Append(" FROM ");
			query.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));

			query.AppendFormat("\r\n LEFT JOIN {0} ",
				SqlCommandHelper.EscapeTableName(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxBusinessPartnerDRTable.NAME));
			query.AppendFormat(" ON {0} = {1}",
				SqlCommandHelper.EscapeTableColumnName(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxBusinessPartnerDRTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxBusinessPartnerDRTable.Columns.CardCode.NAME),
				SqlCommandHelper.EscapeTableColumnName(PmxPickListTable.NAME, PmxPickListTable.Columns.CardCode.NAME));

			query.AppendFormat("\r\n LEFT JOIN {0} AS {1}",
				SqlCommandHelper.EscapeTableName(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxBusinessPartnerDRTable.NAME),
				SqlCommandHelper.EscapeTableName(tableNameInitiator));
			query.AppendFormat(" ON {0} = {1}",
				SqlCommandHelper.EscapeTableColumnName(tableNameInitiator, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxBusinessPartnerDRTable.Columns.CardCode.NAME),
				SqlCommandHelper.EscapeTableColumnName(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxBusinessPartnerDRTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxBusinessPartnerDRTable.Columns.OpdrachtgeverCode.NAME));



			query.Append(" WHERE ");
			SqlCommandHelper.AddTableColumnName(query, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
			query.Append(" = ");
			query.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, dbCulture));

			query.Append(" AND  ");
			SqlCommandHelper.AddTableColumnName(query, tableNameInitiator, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxBusinessPartnerDRTable.Columns.SendDSPFile.NAME);
			query.Append(" = ");
			query.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxMovSendDSPCodes.AFTER_PACKING, dbCulture));


			return query.ToString();
		}



        /// <summary>
        /// Gets the pick list.
        /// </summary>       
        /// <returns></returns>
        public static string GetPickList()
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("SELECT ");
              
            strBuilder.Append(String.Format("{0} AS {1}, "                  , "CardCode"      , PickListDataSet.Columns.CUSTOMER_CODE.Name));
            strBuilder.Append(String.Format("{0} AS {1}, "                  , "CardName"      , PickListDataSet.Columns.CUSTOMER_NAME.Name));
            strBuilder.Append(String.Format("{0} AS {1}, "                  , "DeliveryAdress", PickListDataSet.Columns.CUSTOMER_DELIVERY_ADDRESS.Name));
            strBuilder.Append(String.Format("{0} AS {1}, "                  , "DocEntry"      , PickListDataSet.Columns.DOC_ENTRY.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "DocEntry"      , PickListDataSet.Columns.DOC_ENTRY_AS_STRING.Name));
            strBuilder.Append(String.Format("CAST({0} AS DATETIME) AS {1}, ", "null"          , PickListDataSet.Columns.DUE_DATE.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.DEST_STORLOC_CODE.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.PICK_PACK_REMARKS.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.MOVE_TO_WAREHOUSE_CODE.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.MOVE_TO_WAREHOUSE_NAME.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.SHIPPING_ID.Name));

            strBuilder.Append(String.Format("{0},"                          , "SalesOrderDocEntry"));
            strBuilder.Append(String.Format("{0},"                          , "DestStorLocCode"));
            strBuilder.Append(String.Format("{0},"                          , "ContactName"));
            strBuilder.Append(String.Format("{0},"                          , "ContactTel"));
            strBuilder.Append(String.Format("{0},"                          , "ContactFax"));
            strBuilder.Append(String.Format("{0}"                           , "ContactCellolar"));
            
            strBuilder.Append("\r\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName("PMX_MOV_PACKED_PICKLISTS"));

            return strBuilder.ToString();
        }

        /// <summary>
        /// Gets the customer shipping address.
        /// </summary>       
        /// <returns></returns>
        public static string GetSalesOrderInfo(int pSalesOrderEntry)
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("SELECT ");

            SqlCommandHelper.AddTableColumnName(strBuilder, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.InitiatorCode.NAME); // U_PMX_DR_IMSC
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboShippingTypeTable.NAME, SboShippingTypeTable.Columns.Name.NAME); // TrnspName
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField6.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField7.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField8.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.NumAtCard.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.CardName.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.Address2.NAME);

            strBuilder.Append("\r\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME));
            strBuilder.Append("\r\n LEFT OUTER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboShippingTypeTable.NAME));
            strBuilder.Append(" ON (");
            SqlCommandHelper.AddTableColumnName(strBuilder, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TrnspCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboShippingTypeTable.NAME, SboShippingTypeTable.Columns.Code.NAME);
            strBuilder.Append(") ");
                
            strBuilder.Append("\r\n WHERE ");

            SqlCommandHelper.AddTableColumnName(strBuilder, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.NAME, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.DocEntry.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pSalesOrderEntry, CultureInfo.InvariantCulture));

            return strBuilder.ToString();
        }

        /// <summary>
        /// Gets the customer shipping address.
        /// </summary>       
        /// <returns></returns>
        public static string GetCustomerAddress(int pSalesOrderEntry)
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("SELECT ");

            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.StreetS.NAME, SboSalesOrderTaxExtensionTable.Columns.StreetS.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.BlockS.NAME, SboSalesOrderTaxExtensionTable.Columns.BlockS.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.BuildingS.NAME, SboSalesOrderTaxExtensionTable.Columns.BuildingS.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.CityS.NAME, SboSalesOrderTaxExtensionTable.Columns.CityS.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.ZipCodeS.NAME, SboSalesOrderTaxExtensionTable.Columns.ZipCodeS.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.CountyS.NAME, SboSalesOrderTaxExtensionTable.Columns.CountyS.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.StateS.NAME, SboSalesOrderTaxExtensionTable.Columns.StateS.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.CountryS.NAME, SboSalesOrderTaxExtensionTable.Columns.CountryS.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.StreetNoS.NAME, SboSalesOrderTaxExtensionTable.Columns.StreetNoS.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.Address2S.NAME, SboSalesOrderTaxExtensionTable.Columns.Address2S.NAME);
                         
            strBuilder.Append("\r\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboSalesOrderTaxExtensionTable.NAME));

            strBuilder.Append("\r\n WHERE ");

            SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTaxExtensionTable.NAME, SboSalesOrderTaxExtensionTable.Columns.DocEntry.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pSalesOrderEntry, CultureInfo.InvariantCulture));

            return strBuilder.ToString();
        }


        public static string GetPrincipalLayout(int SalesOrderDocentry)
        {
            return string.Format(@"select [@PMX_MOV_LAYOUTS].Name from [@PMX_MOV_LAYOUTS]
					INNER JOIN ORDR ON [@PMX_MOV_LAYOUTS].Code = ORDR.U_PMX_DR_IMSC
                                       WHERE    (dbo.ORDR.DocEntry= {0}) ", SalesOrderDocentry.ToString());


        }

        /// <summary>
        /// Gets the where clause for the PMX_MOV_PLCD table.
        /// </summary>       
        /// <returns></returns>
        internal static string GetPMX_MOV_PLCDWhereClause(string pColliID, PmxDbConnection dbConn)
        {
            StringBuilder strBuilder = new StringBuilder();

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.ColliID.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pColliID, CultureInfo.InvariantCulture));

            return strBuilder.ToString();
        }

	}


	#endregion

}
