﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using System.Xml;
using System.Web;
using System.Net;
using System.IO;
using System.Threading;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DataSetClient;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.MoviantoNL.Providers;


// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
	/// <summary>
	/// Cycle counting
	/// </summary>
	public class WorkflowScript_TruckLoadingMoviantoScript : WorkflowInstanceScriptBase
	{
        private static class Definitions
        {
            public const string REPORT_CMR    = @"MoviantoCMR.rpt";
        }

		// Input parameters
		public ReadOnlyBinder<CultureInfo>   DefaultCultureInfo;
		public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		private ISboProviderService m_sboProviderService;


		#region Constructors

		public WorkflowScript_TruckLoadingMoviantoScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
			: base(parent, factory)
		{

		}

		#endregion

		#region WorkflowInstanceScriptBase Members

		/// <summary>
		/// Executes this instance.
		/// </summary>
		protected override void Execute()
		{
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
			ProdumexError pmxError = null;

			string initialErrorKey = null;
			string deviceID = session.DeviceInstanceId;
			SboUserInfo userInfo = m_sboProviderService.GetCurrentUser();
			string query = null;

            ToolBarButtons toolBarButtons = ToolBarButtons.Back;



            Message msg                = null;


            int     pickListDocEntry   = 0;
            int     salesorderDocEntry = 0;
            string  location           = String.Empty;
            
            string Carrier = String.Empty;
            bool   CMR     = false;
            string AWBNr   = String.Empty;
            bool   PrintCMR = false;

            string ColliID     = String.Empty;
            string Pallet_SSCC = String.Empty;
            int    Pallet_LUID = 0;

            string buttonKeyPalletConsolidationTask;

            Collection<string> addedSSCCs = null;   
         
            DataSet SSCCAdded = new DataSet();
            SSCCAdded.Tables.Add(new DataTable());
            SSCCAdded.Tables[0].Columns.Add(SSCCDataSet.Columns.SSCC.Name, typeof(System.String));
            SSCCAdded.Tables[0].Columns.Add(SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name, typeof(System.String));
            SSCCAdded.Tables[0].Columns.Add(SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name, typeof(System.String));
            SSCCAdded.Tables[0].Columns.Add(SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name, typeof(System.String));
            SSCCAdded.Tables[0].Columns.Add(SSCCDataSet.Columns.LUID.Name, typeof(System.Int32));
            SSCCAdded.Tables[0].Columns.Add("PRINTCMR", typeof(System.Boolean));



        Step_SelectCarrier:
          
            msg = null;

            query = BuildQuery.GetCarrierList();
            DataSet dsCarrierList = m_sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectEnterStringValueScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection( "TitleKey"       , "Select Carrier",
                                                                               "StringValueDS"  , dsCarrierList,
                                                                               "InitialErrorKey", null,
                                                                               "AllowToGoBack"  , true,
                                                                               "OnlySelectStringValue", true,
                                                                               "AllowEmptyStringValue", false));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".StringValueSelected"))
            {
                Carrier = ExtractParameter<string>(msg.Parameters, "stringValue");

                SSCCAdded.Tables[0].Clear();

                goto Step_ScanPallets;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_End;
            }

        Step_ScanPallets:

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyMultipleSSCCScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection( "TitleKey"           , "Scan Pallets",
                                                                               "SSCCDS"            , SSCCAdded,
                                                                              // "ForceDataEntry"     , true,
                                                                               "InitialErrorKey"    , null,
                                                                               "DisableDeleteButton", false));

            msg = WaitForMessage();

            initialErrorKey = null;
            if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_SelectCarrier;
             
            }
            if (msg.Name.EndsWith("SSCCScanned"))            
            {
                DataRow[] foundRows;
                foundRows = SSCCAdded.Tables[0].Select(String.Format("SSCC = '{0}'", ExtractParameter<string>(msg.Parameters, "sscc")));

                if (foundRows.Length == 0)
                {
                    String  scannedSSCC = ExtractParameter<string>(msg.Parameters, "sscc");
                    DataSet dsPalletIDCarrier = m_sboProviderService.RunView(false, null, null, BuildQuery.GetPalletIDCarrierQuery(scannedSSCC));
                    String  PalletIDCarrier   = dsPalletIDCarrier.Tables[0].Rows.Count == 0 || dsPalletIDCarrier.Tables[0].Rows[0].IsNull("MOV_Carrier") ? String.Empty : dsPalletIDCarrier.Tables[0].Rows[0]["MOV_Carrier"].ToString();

                    if (Carrier == String.Empty || PalletIDCarrier == String.Empty)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                   DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", null,
                                       "MessageKey", "Carrier is leeg.",
                                       "ShowButton", true));

                        msg = WaitForMessage();

                        goto Step_ScanPallets;
                    }

                    if (Carrier == PalletIDCarrier)
                    {
                        int  PmxPalletID_InternalKey = Convert.ToInt32(dsPalletIDCarrier.Tables[0].Rows[0][PmxLogisticUnitIDTable.Columns.InternalKey.NAME]);
                        bool PmxPalletID_PrintCMR    = Convert.ToBoolean(dsPalletIDCarrier.Tables[0].Rows[0][PmxLogisticUnitIDTableMOV.Columns.MOV_CMR.NAME]);

                        DataRow row = SSCCAdded.Tables[0].NewRow();
                        row[SSCCDataSet.Columns.SSCC.Name] = scannedSSCC;
                        row[SSCCDataSet.Columns.LUID.Name] = PmxPalletID_InternalKey;
                        row[SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name] = Carrier;
                        row["PRINTCMR"] = PmxPalletID_PrintCMR;
                        SSCCAdded.Tables[0].Rows.Add(row);

                        goto Step_ScanPallets;
                    }
                    else
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                   DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", null,
                                       "MessageKey", String.Format("Carrier van ingescand palet is niet {0}", Carrier),
                                       "ShowButton", true));

                        msg = WaitForMessage();

                        goto Step_ScanPallets;
                    }
                }
                else
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection(
                                   "InitialErrorKey", null,
                                   "MessageKey", "Pallet is reeds toegevoegd.",
                                   "ShowButton", true));

                    msg = WaitForMessage();

                    goto Step_ScanPallets;
                }
            }
            if (msg.Name.EndsWith("DeleteSSCC"))
            {
                DataRow[] foundRows;
                foundRows = SSCCAdded.Tables[0].Select(String.Format("LUID = {0}", ExtractParameter<int>(msg.Parameters, "luid")));

                if (foundRows.Length == 1)
                    SSCCAdded.Tables[0].Rows.Remove(foundRows[0]);

                goto Step_ScanPallets;
            }
            if (msg.Name.EndsWith("ForwardRequested"))
            {
                goto StepPrintCMR;
            }


        StepPrintCMR:
           
            try
            {
                if (SSCCAdded.Tables[0].Rows.Count == 0)
                    goto Step_End;
                
                query = BuildQuery.GetNewGroupIDFromPMXMOVGSSC();
                DataSet dsNewGroupIDds = m_sboProviderService.RunView(false, null, null, query);


                // Int32   lGroupID  = Convert.ToInt32(SSCCAdded.Tables[0].Rows[0][SSCCDataSet.Columns.LUID.Name]);
                Int32 lGroupID = Convert.ToInt32(dsNewGroupIDds.Tables[0].Rows[0][0]);
                Boolean lPrintCMR = false;

                // WRITE ROWS TO NEW USER TABLE
                m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, delegate(PmxDbConnection conn, object[] parameters)
                {
                    PmxMovGSSCProvider prov = new PmxMovGSSCProvider(conn);
                  
                    foreach (DataRow lRow in SSCCAdded.Tables[0].Rows)
                    {
                        if (Convert.ToBoolean(lRow["PRINTCMR"]))
                            lPrintCMR = true;

                        using (PmxMovGSSC obj = prov.GetNewBO())
                        {
                            obj.GroupID = lGroupID;
                            obj.LUID    = Convert.ToInt32(lRow[SSCCDataSet.Columns.LUID.Name]);

                            prov.AddBO(obj);
                        }
                    }

                    return null;
                });

                if (lPrintCMR)
                {
                    m_sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintCMRReport), lGroupID, session.DeviceInstanceId, Definitions.REPORT_CMR);
                    Thread.Sleep(750);
                }
            }
            catch (Exception ex)
            {
                s_log.Error(ex.ToString());

                if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }

              //  throw ex;
            }

            goto Step_CreateDeliveries;


        Step_CreateDeliveries:

            try
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                   this.DefaultCultureInfo, BuildParamCollection(
                   "MessageKey", "MSG_ADDING_SALES_DELIVERIES"));


                Collection<int> listOfPackedPickLists = new Collection<int>();
                Dictionary<string, object> deliveryData = new Dictionary<string, object>();

                List<int> lLUIDs = new List<int>();
                foreach (DataRow lRow in SSCCAdded.Tables[0].Rows)
                    lLUIDs.Add(Convert.ToInt32(lRow[SSCCDataSet.Columns.LUID.Name]));

                DataSet dsPicklists = m_sboProviderService.RunView(false, null, null, BuildQuery.GetPicklistDocEntriesForPalletIDs(lLUIDs.ToArray()));

                foreach (DataRow lPicklistrow in dsPicklists.Tables[0].Rows)
                    listOfPackedPickLists.Add(Convert.ToInt32(lPicklistrow[PmxMovPLCDTable.Columns.PickListDocEntry.NAME]));

                m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null, delegate(PmxDbConnection conn, object[] parameters)
                {
                    PmxSalesDeliveryNoteProvider prov = new PmxSalesDeliveryNoteProvider(conn);

                    prov.ShipAllItemsForPickList(listOfPackedPickLists, null, null);

                    return null;
                });

                goto Step_End;
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //retry?
                //ask user if he wants to retry
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_RETRY_UPDATE",
                        "MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
                msg = WaitForMessage();

                if (msg.Name.ToLower().EndsWith(".yes"))
                {
                    goto Step_CreateDeliveries;
                }

                goto Step_End;

            }

        Step_End:

            pickListDocEntry = 0;
		}

		#endregion


		#region HELPERS

		/// <summary>
		/// Executes the get PMX warehouse info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxOseProvider oseProv = new PmxOseProvider(conn);
			return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
		}

		/// <summary>
		/// Gets the number of rows.
		/// </summary>
		/// <param name="ds">The ds.</param>
		/// <returns></returns>
		private int GetNumberOfRows(DataSet ds)
		{
			if (ds != null &&
				ds.Tables.Count > 0)
			{
				return ds.Tables[0].Rows.Count;
			}

			return 0;
		}



		/// <summary>
		/// Executes the get item info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
			return prov.GetCachedItemInfo((string)parameters[0]);
		}

        /// <summary>
        /// Executes the report Pallet Label.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintCMRReport(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@GROUPID", (int)parameters[0]));
            
            // TODO : get location
            return prov.PrintReport(reportParameters, (string)parameters[2], 1, (string)parameters[1], null, false);
        }

        /// <summary>
        /// Executes the report.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static object AddData(PmxDbConnection conn, object[] parameters)
        {
            PmxLogisticUnitIDProvider prov = new PmxLogisticUnitIDProvider(conn);

            using (PmxLogisticUnitID luid = prov.GetNewBO())
            {
                luid.IsFullPallet = true;

                prov.AddBO(luid);
 
            }


            return null;
        }

		#endregion

	}

	#region BUILD QUERY

	/// <summary>
	/// Static class grouping our native queries
	/// </summary>
	internal static class BuildQuery
	{
        /// <summary>
        /// Gets the carrier list.
        /// </summary>       
        /// <returns></returns>
        public static string GetCarrierList () 
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("SELECT ");

            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, "@PMX_MOV_CARRI", "Name",  StringValueDataSet.Columns.STRING_VALUE.Name);

            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName("@PMX_MOV_CARRI"));

            return strBuilder.ToString();
        }

        public static string GetPicklistDocEntriesForPalletIDs(int[] pPalletIDs)
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("SELECT DISTINCT ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.PickListDocEntry.NAME);

            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMovPLCDTable.NAME));
            strBuilder.Append(" WHERE ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.PalletID.NAME);
            strBuilder.Append(" IN (");

            String lIDS = String.Empty;
            foreach(int lPalletID in pPalletIDs)
                lIDS += lIDS == (String.Empty) ? (SqlCommandHelper.FormatAndEscapeColumnValue(lPalletID, CultureInfo.InvariantCulture)) : "," + (SqlCommandHelper.FormatAndEscapeColumnValue(lPalletID, CultureInfo.InvariantCulture));

            strBuilder.Append(lIDS);
            strBuilder.Append(")");

            return strBuilder.ToString();             
        }

        public static string GetPalletIDCarrierQuery (string pPalletSSCC) 
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT TOP 1");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTableMOV.Columns.MOV_Carrier.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTableMOV.Columns.MOV_CMR.NAME);
            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
            strBuilder.Append(" WHERE ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pPalletSSCC, CultureInfo.InvariantCulture));

            strBuilder.Append(" ORDER BY ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
            strBuilder.Append(" DESC ");

            return strBuilder.ToString();
        }

        public static string GetNewGroupIDFromPMXMOVGSSC()
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("select coalesce(MAX(");
            strBuilder.Append(PmxMovGSSCTable.Columns.GroupID.NAME);
            strBuilder.Append("),0) + 1");
            strBuilder.Append(" FROM ");
            strBuilder.Append(PmxMovGSSCTable.NAME);

            return strBuilder.ToString();   
        }
	}


	#endregion

}
